// 搜索选择器，配置字典
import React, { useState, useMemo } from 'react'
import { Select } from 'antd'
import { getSelectName } from '../../../../apis/process'

const { Option } = Select

const SearchSelect = ({ value, onChange, schema }) => {

  const { disabled, placeholder } = schema
  const [selectList, setSelectList] = useState([]);

  const mode = useMemo(() => {
    if (schema.config.includes('n')) {
      return 'multiple'
    }
    return null
  }, [schema])

  const valueShow = useMemo(() => {
    if (mode) {
      return (schema.default || value) ? (schema.default || value).split(',') : []
    } else {
      return schema.default || value
    }
  }, [schema, value])

  const handleChange = value => {
    if (mode) {
      onChange(value.join(','))
    } else {
      onChange(value)
    }
  }

  const getData = () => {
    const arr = schema.config.split('.')
    if (!arr.length) return false
    const str = arr[0];
    getSelectName(str).then(res => {
      if (res.code === 0) {
        setSelectList(res.data)
      } else {
        onChange('请求错误')
      }
    })
  }

  return (
    <Select
      mode={mode}
      disabled={disabled}
      showArrow={!disabled}
      optionFilterProp="children"
      value={valueShow || null}
      placeholder={placeholder}
      onChange={handleChange}
      onFocus={() => getData()}
      style={{ width: '100%' }}
      showSearch
    >
      {selectList.map(v => <Option value={v.nodeValue} key={v.nodeId}>{v.nodeValue}</Option>)}
    </Select>
  )
}

export default SearchSelect
