// 台账选择器
/*
 ** 台账选择器
 ** 功能路径：src\core\components\tableAccount\tableAccount.js
 ** 菜单参数列表：*变量名*(变量说明，数据类型，是否必填，取值范围)
 ** 更新日志：
 ** edit by ChenLong on 2021/12/23: ***** 1.修改无法点击选择的bug；2.现在初始化状态下，不显示其他台账；3.文件类字段不显示 *****
 ** edit by ChenLong on 2022/02/28: ***** 1.使用btoa来给condition转码;转码 window.btoa(encodeURIComponent('aaa'))；恢复 decodeURIComponent(window.atob('bbb')) *****
 ** edit by ChenLong on 2022/08/02: ***** 1.台账接口需要sortFields才能开启排序 *****
 */

import React, { useState } from 'react';
import { Modal, Button, Radio, Input, Table, Space, ConfigProvider, message } from 'antd';
import BasicTable from '@wisdom-components/basictable';
import { SnippetsOutlined } from '@ant-design/icons';
import zhCN from 'antd/es/locale/zh_CN';
import Empty from '@wisdom-components/empty/lib';
import { GetAccountConfigInfo, GetAccountPageList } from '../../../../apis/process';
import './index.less';


const { Column } = Table;
const { Search } = Input;
const FILE_TYPE = ['文件', '可预览文件', '图片', '可预览图片', '录音', '视频'];

class TableAccount extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      visible: false,
      columns: [],
      tableData: [],
      tableAccountValue: '',
      searchVal: '',
      loading: true,
      pagination: {
        current: 1,
        pageSize: 20,
        total: 0,
        size: 'default',
      },
      isInit: true,
      selectionType: 'radio',
      selectedRowKeys: [...props.value.split(',')],
      rowSelection: {
        onChange: (selectedRowKeys, record) => {
          console.log('selectedRowKeys', selectedRowKeys)
          this.setState({
            tableAccountValue: selectedRowKeys.join(','),
            showString: record.map(item => item[this.state.showKey]).join(','),
            selectedRowKeys,
          });
        },
      },
      showString: props.value, //edit by Chenlong 2022年10月28日 修复初始加载无法填充值的问题
      accountName: '',
      condition: '',
      // 新增rowKey、showKey
      rowKey: '',
      showKey: '',
      // 用showKey来显示
    };
  }

  componentDidMount() {
    this.handleConfig(
      () => {
        this.getAccountConfig(
          () => this.getAccountPageListFn(this.state.info, this.state.info ? '' : this.state.condition),
        );
      },
    );
  }

  returnSelectType(selectionTypeStr) {
    const typeObj = {
      '台账选择器': 'radio',
      '多选台账选择器': 'checkbox',
    };
    return typeObj[selectionTypeStr];
  }

  /**
   *  @Description: 获取配置
   *  @Params: 传递callback，看情况是否需要调用
   *  @Date: 2021/12/27
   * */
  getAccountConfig(callback) {
    GetAccountConfigInfo(this.state.accountName).then(res => {
      if (res.code === 0) {
        const colArr = [];
        const arr = res.data.webShowFieldGroup.split(',');
        arr.forEach(item => {
          if (!item) {
            return;
          }
          colArr.push({
            key: new Date().getTime(),
            title: item,
            dataIndex: item,
          });
        });
        this.setState(
          {
            columns: colArr,
            total: res.data.totalCount,
          }, () => {
            if (callback) {
              callback();
            }
          });
      } else {
        this.props.onChange(`出错了！${res.msg}`);
      }
    });
  }
  async getAccountPageListFn(info, condition) {
    this.setState({
      loading: true,
    });
    const arrWrap = [];
    let showStringArray = [];

    let getAllData = async () => {
      let _infoList = info.split(',');
      await Promise.all(_infoList.map(async currentInfo => {
        await GetAccountPageList({
          pageIndex: this.state.pagination.current,
          pageSize: this.state.pagination.pageSize,
          accountName: this.state.accountName,
          sortFields: '录入时间',
          info: currentInfo,
          condition: condition ? window.btoa(encodeURIComponent(condition)) : ''
        }).then(
          response => {
            if (response.code === 0) {
              response.data.list.forEach(item => {
                const obj = this.handleItem(item);
                obj.key = obj[this.state.rowKey];
                if (this.state.isInit && this.state.info) {
                  if (currentInfo === obj[this.state.rowKey]) {
                    arrWrap.push(obj);
                    showStringArray.push(obj[this.state.showKey]);
                  }
                } else {
                  arrWrap.push(obj);
                }
              });
            } else {
              this.props.onChange(`出错了！${response.msg}`);
            }
          },
        ).catch(err => {
          this.setState({
            loading: false,
          });
        });
      }));
    };
    await getAllData();
    this.setState({
      loading: false,
      tableData: arrWrap,
      showString: showStringArray.join(','),
    });
  }

  handleConfig(callback) {
    // 判断是否有 ?
    const _configArray = this.props.schema.config.split('?');
    const condition = _configArray[1];
    // 台账名称.台账字段|显示字段.n
    const _accountInfo = _configArray[0].split('.');
    // 是否多选
    this.setState({
      selectionType: _accountInfo.includes('n') ? 'checkbox' : 'radio',
    });
    // 台账名称
    const accountName = _accountInfo[0];
    let rowKey = '';
    let showKey = '';
    if (_accountInfo[1] && _accountInfo[1].indexOf('|') > -1) {
      const keyArr = _accountInfo[1].split('|');
      rowKey = keyArr[0];
      showKey = keyArr[1];
    } else {
      rowKey = _accountInfo[1];
      showKey = _accountInfo[1];
    }
    // this.state.searchVal || 初始化时候的值
    const info = this.props.value || this.state.searchVal || '';
    this.setState({
      accountName,
      rowKey,
      showKey,
      condition,
      info,
    }, () => {
      if (callback) callback();
    });
  }

  handleItem(obj) {
    const dataObj = obj;
    const objarr = [];
    const objobj = {};
    let fieldValue = [];
    dataObj.webRow.forEach((item, index) => {
      if (FILE_TYPE.includes(item.shape)) {
        fieldValue.push(item.fieldName);
      } else {
        objarr.push(item.fieldValue);
      }
    });
    let _columns = [...this.state.columns];
    _columns = _columns.filter(item => !fieldValue.includes(item.dataIndex));
    for (let i = 0; i < _columns.length; i++) {
      for (let j = 0; j < objarr.length; j++) {
        if (i === j) {
          objobj[_columns[i].dataIndex] = objarr[j];
        }
      }
    }
    this.setState({
      columns: _columns,
    });
    return objobj;
  }

  handleInputClick(e) {
    this.setState({
      visible: true,
    }, () => {
      if (this.state.tableData.length === 0) {

      }
    });
  }

  onOk() {
    this.setState(
      {
        visible: false,
        searchVal: '',
      },
      () => {
        this.props.onChange(this.state.tableAccountValue);
      },
    );
  }

  onCancel() {
    this.setState({
      visible: false,
      searchVal: '',
      isInit: true,
    });
  }

  onSearch(val) {
    this.setState(
      {
        searchVal: val,
      },
      () => {
        this.getAccountPageListFn(val, this.state.condition);
      },
    );
  }

  handleInputChange(e) {
    console.log(e);
  }

  tableChange(pagination) {
    this.setState(
      {
        pagination,
      },
      () => {
        this.getAccountPageListFn(this.state.searchVal, this.state.condition);
      },
    );
  }

  render() {
    return (
      <div className="tableAccount-wrapper" style={{ width: '100%' }} ref={'modal'}>
        <Input
          disabled={this.props.schema.disabled}
          value={this.state.showString}
          placeholder={this.props.schema.disabled ? null : '点击选择台账'}
          addonAfter={
            !this.props.schema.disabled ? <SnippetsOutlined style={{ color: 'rgba(0, 0, 0, 0.25)' }}
              onClick={this.handleInputClick.bind(this)} /> : ''
          }
        />
        <Modal
          title="台账信息"
          visible={this.state.visible}
          onCancel={this.onCancel.bind(this)}
          onOk={this.onOk.bind(this)}
          cancelText={'取消'}
          okText={'确定'}
          width={'60%'}
          bodyStyle={{ height: 505, overflowY: 'auto' }}
          wrapClassName="tableAccountClass"
          destroyOnClose
        >
          {
            this.state.isInit ?
              <Button style={{ width: 120, marginBottom: 20 }} type={'primary'}
                onClick={() => this.setState(({ isInit: false }))}>点击搜索、更换</Button> :
              <Search
                placeholder="请输入检索内容"
                allowClear
                onSearch={this.onSearch.bind(this)}
                enterButton
                size={'middle'}
                style={{ width: '400px', marginBottom: '10px' }}
              />
          }
          {!this.state.columns.length && (
            <div className={'tableAccountEmpty'}>
              <Empty />
            </div>
          ) || <ConfigProvider locale={zhCN}>
              {/* <BasicTable
              rowSelection={{
                type: this.state.selectionType,
                ...this.state.rowSelection,
                selectedRowKeys: this.state.selectedRowKeys,
                fixed: 'left',
              }}
              bordered
              size="small"
              columns={this.state.columns}
              dataSource={this.state.tableData}
              rowClassName="rowClassName"
              style={{ width: '100%' }}
              sticky
              loading={this.state.loading}
              scroll={{ x: 1500, y: 340 }}
              pagination={this.state.pagination}
              onChange={this.tableChange.bind(this)}
            />   */}
              <Table
                rowSelection={{
                  type: this.state.selectionType,
                  ...this.state.rowSelection,
                  selectedRowKeys: this.state.selectedRowKeys,
                  fixed: 'left',
                }}
                bordered
                size="small"
                columns={this.state.columns}
                dataSource={this.state.tableData}
                rowClassName="rowClassName"
                style={{ width: '100%' }}
                sticky
                loading={this.state.loading}
                scroll={{ x: 1500, y: 340 }}
                pagination={this.state.pagination}
                onChange={this.tableChange.bind(this)}
              />
            </ConfigProvider>}
        </Modal>
      </div>
    );
  }
}

export default TableAccount;
